@@ -1,6 +1,6 @@
 ''' Agent RTM client implementation. '''
 
-# pylint: disable=W0613,W0622,C0103,R0913,R0903,W0107
+# pylint: disable=W0613,W0622,C0103,R0913,R0903,W0107,W0221
 
 import typing
 from abc import ABCMeta
@@ -26,7 +26,10 @@ class AgentRTM:
             Raises:
                 ValueError: If the specified version does not exist.
         '''
-        client = {'3.3': AgentRTM33(version, base_url)}.get(version)
+        client = {
+            '3.3': AgentRTM33(version, base_url),
+            '3.4': AgentRTM34(version, base_url)
+        }.get(version)
         if not client:
             raise ValueError('Provided version does not exist.')
         return client
@@ -47,6 +50,31 @@ class AgentRTMInterface(metaclass=ABCMeta):
 
 # Chats
 
+    def add_user_to_chat(self,
+                         chat_id: str = None,
+                         user_id: str = None,
+                         user_type: str = None,
+                         require_active_thread: bool = None,
+                         payload: dict = None) -> dict:
+        ''' Adds a user to the chat. You can't add more than
+            one customer user type to the chat.
+
+            Args:
+                chat_id (str): Chat ID.
+                user_id (str): ID of the user that will be added to the chat.
+                user_type (str): Possible values: agent or customer.
+                require_active_thread (bool): A flag. If True, it adds a user to a chat
+                    only if that chat has an active thread; default False.
+                payload (dict): Custom payload to be used as request's data.
+                        It overrides all other parameters provided for the method.
+
+            Returns:
+                dict: Dictionary with response.
+        '''
+        if payload is None:
+            payload = prepare_payload(locals())
+        return self.ws.send({'action': 'add_user_to_chat', 'payload': payload})
+
     def list_chats(self,
                    filters: dict = None,
                    sort_order: str = None,
@@ -236,50 +264,6 @@ class AgentRTMInterface(metaclass=ABCMeta):
 
 # Chat access
 
-    def grant_chat_access(self,
-                          id: str = None,
-                          access: dict = None,
-                          payload: dict = None) -> dict:
-        ''' Grants access to a new chat without overwriting the existing ones.
-
-            Args:
-                id (str): Chat ID.
-                access (dict): Access object.
-                payload (dict): Custom payload to be used as request's data.
-                        It overrides all other parameters provided for the method.
-
-            Returns:
-                dict: Dictionary with response.
-        '''
-        if payload is None:
-            payload = prepare_payload(locals())
-        return self.ws.send({
-            'action': 'grant_chat_access',
-            'payload': payload
-        })
-
-    def revoke_chat_access(self,
-                           id: str = None,
-                           access: dict = None,
-                           payload: dict = None) -> dict:
-        ''' Revokes access to a chat.
-
-            Args:
-                id (str): Chat ID.
-                access (dict): Access object.
-                payload (dict): Custom payload to be used as request's data.
-                        It overrides all other parameters provided for the method.
-
-            Returns:
-                dict: Dictionary with response.
-        '''
-        if payload is None:
-            payload = prepare_payload(locals())
-        return self.ws.send({
-            'action': 'revoke_chat_access',
-            'payload': payload
-        })
-
     def transfer_chat(self,
                       id: str = None,
                       target: dict = None,
@@ -306,31 +290,6 @@ class AgentRTMInterface(metaclass=ABCMeta):
 
 # Chat users
 
-    def add_user_to_chat(self,
-                         chat_id: str = None,
-                         user_id: str = None,
-                         user_type: str = None,
-                         require_active_thread: bool = None,
-                         payload: dict = None) -> dict:
-        ''' Adds a user to the chat. You can't add more than
-            one customer user type to the chat.
-
-            Args:
-                chat_id (str): Chat ID.
-                user_id (str): ID of the user that will be added to the chat.
-                user_type (str): Possible values: agent or customer.
-                require_active_thread (bool): A flag. If True, it adds a user to a chat
-                    only if that chat has an active thread; default False.
-                payload (dict): Custom payload to be used as request's data.
-                        It overrides all other parameters provided for the method.
-
-            Returns:
-                dict: Dictionary with response.
-        '''
-        if payload is None:
-            payload = prepare_payload(locals())
-        return self.ws.send({'action': 'add_user_to_chat', 'payload': payload})
-
     def remove_user_from_chat(self,
                               chat_id: str = None,
                               user_id: str = None,
@@ -967,4 +926,189 @@ class AgentRTMInterface(metaclass=ABCMeta):
 
 
 class AgentRTM33(AgentRTMInterface):
-    ''' AgentRTM version 3.3 class. '''
+    ''' Agent RTM version 3.3 class. '''
+
+    # Chat access
+
+    def grant_chat_access(self,
+                          id: str = None,
+                          access: dict = None,
+                          payload: dict = None) -> dict:
+        ''' Grants access to a new chat without overwriting the existing ones.
+
+            Args:
+                id (str): Chat ID.
+                access (dict): Access object.
+                payload (dict): Custom payload to be used as request's data.
+                        It overrides all other parameters provided for the method.
+
+            Returns:
+                dict: Dictionary with response.
+        '''
+        if payload is None:
+            payload = prepare_payload(locals())
+        return self.ws.send({
+            'action': 'grant_chat_access',
+            'payload': payload
+        })
+
+    def revoke_chat_access(self,
+                           id: str = None,
+                           access: dict = None,
+                           payload: dict = None) -> dict:
+        ''' Revokes access to a chat.
+
+            Args:
+                id (str): Chat ID.
+                access (dict): Access object.
+                payload (dict): Custom payload to be used as request's data.
+                        It overrides all other parameters provided for the method.
+
+            Returns:
+                dict: Dictionary with response.
+        '''
+        if payload is None:
+            payload = prepare_payload(locals())
+        return self.ws.send({
+            'action': 'revoke_chat_access',
+            'payload': payload
+        })
+
+
+class AgentRTM34(AgentRTMInterface):
+    ''' Agent RTM version 3.4 class. '''
+
+    # Chats
+
+    def add_user_to_chat(self,
+                         chat_id: str = None,
+                         user_id: str = None,
+                         user_type: str = None,
+                         visibility: str = None,
+                         ignore_requester_presence: bool = None,
+                         payload: dict = None) -> dict:
+        ''' Adds a user to the chat. You can't add more than
+            one customer user type to the chat.
+
+            Args:
+                chat_id (str): Chat ID.
+                user_id (str): ID of the user that will be added to the chat.
+                user_type (str): Possible values: agent or customer.
+                visibility (str): Determines the visibility of events sent by
+                                  the agent. Possible values: `all` or `agents`.
+                ignore_requester_presence (bool): If `True`, allows requester to add user to chat
+                                                  without being present in the chat's users list.
+                payload (dict): Custom payload to be used as request's data.
+                                It overrides all other parameters provided for
+                                the method.
+
+            Returns:
+                dict: Dictionary with response.
+        '''
+        if payload is None:
+            payload = prepare_payload(locals())
+        return self.ws.send({'action': 'add_user_to_chat', 'payload': payload})
+
+    def deactivate_chat(self,
+                        id: str = None,
+                        ignore_requester_presence: bool = None,
+                        payload: dict = None) -> dict:
+        ''' Deactivates a chat by closing the currently open thread.
+
+            Args:
+                id (str): Chat ID to deactivate.
+                ignore_requester_presence (bool): If `True`, allows requester to deactivate chat
+                                                  without being present in the chat's users list.
+                payload (dict): Custom payload to be used as request's data.
+                        It overrides all other parameters provided for the method.
+
+            Returns:
+                dict: Dictionary with response.
+        '''
+        if payload is None:
+            payload = prepare_payload(locals())
+        return self.ws.send({'action': 'deactivate_chat', 'payload': payload})
+
+# Chat access
+
+    def transfer_chat(self,
+                      id: str = None,
+                      target: dict = None,
+                      ignore_agents_availability: bool = None,
+                      ignore_requester_presence: bool = None,
+                      payload: dict = None) -> dict:
+        ''' Transfers a chat to an agent or a group.
+
+            Args:
+                id (str): Chat ID.
+                target (dict): Target object. If missing, the chat will be
+                        transferred within the current group.
+                ignore_agents_availability (bool): If `True`, always transfers chats. Otherwise, fails
+                              when unable to assign any agent from the requested groups.
+                ignore_requester_presence (bool): If `True`, allows requester to transfer chat
+                                                  without being present in the chat's users list.
+                payload (dict): Custom payload to be used as request's data.
+                        It overrides all other parameters provided for the method.
+
+            Returns:
+                dict: Dictionary with response.
+        '''
+        if payload is None:
+            payload = prepare_payload(locals())
+        return self.ws.send({'action': 'transfer_chat', 'payload': payload})
+
+# Chat users
+
+    def remove_user_from_chat(self,
+                              chat_id: str = None,
+                              user_id: str = None,
+                              user_type: str = None,
+                              ignore_requester_presence: bool = None,
+                              payload: dict = None) -> dict:
+        ''' Removes a user from chat.
+
+            Args:
+                chat_id (str): Chat ID.
+                user_id (str): ID of the user that will be added to the chat.
+                user_type (str): Possible values: agent or customer.
+                ignore_requester_presence (bool): If `True`, allows requester to remove user from chat
+                                                  without being present in the chat's users list.
+                payload (dict): Custom payload to be used as request's data.
+                        It overrides all other parameters provided for the method.
+
+            Returns:
+                dict: Dictionary with response.
+        '''
+        if payload is None:
+            payload = prepare_payload(locals())
+        return self.ws.send({
+            'action': 'remove_user_from_chat',
+            'payload': payload
+        })
+
+
+# Other
+
+    def send_typing_indicator(self,
+                              chat_id: str = None,
+                              visibility: str = None,
+                              is_typing: bool = None,
+                              payload: dict = None) -> dict:
+        ''' Sends a typing indicator.
+
+            Args:
+                chat_id (str): ID of the chat you want to send the typing indicator to.
+                visibility (str): Possible values: `all`, `agents`.
+                is_typing (bool): A flag that indicates if you are typing.
+                payload (dict): Custom payload to be used as request's data.
+                        It overrides all other parameters provided for the method.
+
+            Returns:
+                dict: Dictionary with response.
+        '''
+        if payload is None:
+            payload = prepare_payload(locals())
+        return self.ws.send({
+            'action': 'send_typing_indicator',
+            'payload': payload
+        })
